use serde_derive::{Serialize, Deserialize};
use std::path::PathBuf;
use std::fs::read_to_string;

#[derive(Debug, Eq, PartialEq, Serialize, Deserialize)]
pub struct Tile {
    pub name: String,
    /// these will animate
    /// todo should there be animation options? reverse, random, etc?
    /// todo do we need a "current frame" property or leave that up to the player implementation?
    pub images: Vec<String>,
    pub wall: bool,
}

impl Tile {
    pub fn from_file(path: PathBuf) -> Tile {
        let name = path.file_stem().unwrap().to_str().unwrap().into();

        let intermediate: IntermediateTile = toml::from_str(
            &read_to_string(path).unwrap()
        ).unwrap();

        Tile { name, images: intermediate.images, wall: intermediate.wall }
    }
}

#[derive(Debug, Eq, PartialEq, Serialize, Deserialize)]
struct IntermediateTile {
    images: Vec<String>,
    wall: bool,
}

#[cfg(test)]
mod test {
    use std::path::PathBuf;
    use crate::tile::Tile;

    #[test]
    fn from_file() {
        let path = PathBuf::from("src/test-resources/basic/tiles/block.toml");
        let output = Tile::from_file(path);

        let expected = Tile {
            name: "block".into(),
            images: vec!["block".to_string()],
            wall: false
        };

        assert_eq!(output, expected);
    }
}
